' //***************************************************************************
' //
' // File: BFPartUtility.vbs
' //
' // Purpose: 	Update the .imageinfo file generated during an image capture to include
' //			detailed information about the partitions included in the captured
' //			WIM file.
' //
' //***************************************************************************


Option Explicit

Dim bfPartitionsFromOS, bfPartitionsFromWinPE, partitionInfoPath
Set bfPartitionsFromOS = Nothing
Set bfPartitionsFromWinPE = Nothing

Function InitPartitionsFromOS
	Dim partitionString, parts, part
	
	oLogging.CreateEntry "Reading OS partition data from: " & partitionInfoPath, LogTypeInfo

	Set bfPartitionsFromOS = CreateObject("Scripting.Dictionary")
	
	' Read the partition data
	partitionString = oUtility.ReadIni(partitionInfoPath, "OSPartitions", "PartitionData")
	
	oLogging.CreateEntry "Partition string is: '" & partitionString & "'", LogTypeInfo
	
	' split the partitions
	parts = split(partitionString,";")
	
	' store data for each partition
	For each part in parts
		
		Dim components
		
		components = split(part, ":")
		
		bfPartitionsFromOS.Add components(1) & ":" & components(2), components(0) & ":" & components(3) & ":" & components(4)
		
	Next

	oLogging.CreateEntry "Completed reading partition data from the OS", LogTypeInfo
	
End Function

Function InitPartitionsFromWinPE
	Dim partition, partitions, components

	oLogging.CreateEntry "Reading WinPE partition data", LogTypeInfo

	Set bfPartitionsFromWinPE = CreateObject("Scripting.Dictionary")
	
	partitions = Empty
	on error resume next
	partitions = oUtility.BDDUtility.HiddenPartitionsToDrives
	on error goto 0 

	For each partition in partitions
		oLogging.CreateEntry "Processing entry: '" & partition & "'", LogTypeInfo

		components = split(partition,vbTab)
		
		If ubound(components) >= 7 then
			' Store the letter as the dictionary key and the disk:partition as the value
			If components(7) <> "" then
				bfPartitionsFromWinPE.Add left(components(7),1), components(1) & ":" & components(2) 
			End if
		Else
			oLogging.CreateEntry "Unable to process entry", LogTypeInfo
		End if
	Next

	oLogging.CreateEntry "Completed reading partition data from WinPE", LogTypeInfo
End Function


Function UpdateImageinfoPartitions ( partition )
	
	Dim partitionInfoData, partitionLetter, partitionUsedSpace, partitionsUsedSpace, partLocation, osPartInfo
	
	partitionInfoPath = oEnvironment.Item("DeployRoot") & "\$OEM$\BigFixOSD\partition.info"

	partitionLetter = partition.DriveLetter
	partitionUsedSpace = (partition.TotalSize - partition.FreeSpace)

	oLogging.CreateEntry "Started processing partition: " & partitionLetter, LogTypeInfo

	' If the list of partitions from the OS has not been initialized, let's read them
	If (bfPartitionsFromOS Is Nothing) or (bfPartitionsFromWinPE Is Nothing) then
		InitPartitionsFromOS
		InitPartitionsFromWinPE
	End if
	
	partLocation = bfPartitionsFromWinPE(partitionLetter)
	
	If partLocation <> "" then
		osPartInfo = bfPartitionsFromOS(partLocation)
		
		If osPartInfo <> "" then
			Dim partitions
			
			' If we are capturing a single partition, this partition is for sure the system one. But we have to force it to
			' be also the boot one
			If Ucase(oEnvironment.Item("BackupDrive")) <> "ALL" then
				Dim components
				
				oLogging.CreateEntry "Fixing partition: " & partLocation & " with info: " & osPartInfo & " to be bootable", LogTypeInfo

				components = split(osPartInfo, ":")
				
				osPartInfo = components(0) & ":" & "T" & ":" & components(2)

				oLogging.CreateEntry "Fixed partition: " & partLocation & " with info: " & osPartInfo, LogTypeInfo
			End if
			
			partitionInfoData   = oUtility.ReadIni(partitionInfoPath, "Partitions", "PartitionData")
			partitionsUsedSpace = oUtility.ReadIni(partitionInfoPath, "Partitions", "PartitionsUsedSpace")
			' The first time, partition used space will be an empty string. So change it to a zero value
			If partitionsUsedSpace = "" then
				partitionsUsedSpace = 0
			End if
			
			oLogging.CreateEntry "Current partition data: " & partitionInfoData & " with size: " & partitionsUsedSpace, LogTypeInfo

			partitions = split(partitionInfoData, ";")
			
			If partitionInfoData <> "" then
				partitionInfoData = partitionInfoData & ";"
			End if
			
			partitionInfoData = partitionInfoData & (ubound(partitions) + 2) & ":" & osPartInfo & ":" & partitionUsedSpace
			partitionsUsedSpace = partitionsUsedSpace + partitionUsedSpace
			
			oLogging.CreateEntry "Updated partition data: " & partitionInfoData & " with size: " & partitionUsedSpace, LogTypeInfo

			oUtility.WriteIni partitionInfoPath, "Partitions", "PartitionData", partitionInfoData
			oUtility.WriteIni partitionInfoPath, "Partitions", "PartitionsUsedSpace", partitionsUsedSpace
		Else
			oLogging.CreateEntry "Unable to find OS data for partition: " & partitionLetter & ", location: " & partLocation, LogTypeInfo
		End if
	Else
		oLogging.CreateEntry "Unable to find WinPE data for partition: " & partitionLetter, LogTypeInfo
	End if
	
End function